#include "sys/Syscall.h"
#include "io/Serial.h"
#include "memory/VMM.h"
#include "std/errno.h"
#include "std/string.h"
#include "thread/Scheduler.h"

void Syscall::entry(Context* context)
{
    asm volatile("cli");
    VMM::enter_syscall_context();
    switch (context->rax)
    {
    case SYS_exit: sys_exit(context, (int)context->rdi); break;
    case SYS_yield: sys_yield(context); break;
    case SYS_sleep: sys_sleep(context, context->rdi); break;
    case SYS_write: sys_write(context, (int)context->rdi, context->rsi, (const char*)context->rdx); break;
    case SYS_paint: sys_paint(context, context->rdi, context->rsi, context->rdx, context->r10, context->r8); break;
    case SYS_getprocid: sys_getprocid(context, (int)context->rdi); break;
    case SYS_mmap: sys_mmap(context, (void*)context->rdi, context->rsi, (int)context->rdx); break;
    case SYS_munmap: sys_munmap(context, (void*)context->rdi, context->rsi); break;
    case SYS_open: sys_open(context, (const char*)context->rdi, (int)context->rsi, (mode_t)context->rdx); break;
    case SYS_read: sys_read(context, (int)context->rdi, context->rsi, (char*)context->rdx); break;
    case SYS_close: sys_close(context, (int)context->rdi); break;
    case SYS_seek: sys_seek(context, (int)context->rdi, (long)context->rsi, (int)context->rdx); break;
    case SYS_execv: sys_execv(context, (const char*)context->rdi, (char**)context->rsi); break;
    case SYS_fcntl: sys_fcntl(context, (int)context->rdi, (int)context->rsi, context->rdx); break;
    case SYS_mprotect: sys_mprotect(context, (void*)context->rdi, context->rsi, (int)context->rdx); break;
    case SYS_clock_gettime: sys_clock_gettime(context, (int)context->rdi, (struct timespec*)context->rsi); break;
    case SYS_mkdir: sys_mkdir(context, (const char*)context->rdi, (mode_t)context->rsi); break;
    case SYS_fork: sys_fork(context); break;
    case SYS_waitpid: sys_waitpid(context, (long)context->rdi, (int*)context->rsi, (int)context->rdx); break;
    case SYS_access: sys_access(context, (const char*)context->rdi, (int)context->rsi); break;
    case SYS_fstat: sys_fstat(context, (int)context->rdi, (struct stat*)context->rsi); break;
    case SYS_stat: sys_stat(context, (const char*)context->rdi, (struct stat*)context->rsi); break;
    case SYS_pstat: sys_pstat(context, (long)context->rdi, (struct pstat*)context->rsi); break;
    case SYS_getdents:
        sys_getdents(context, (int)context->rdi, (struct luna_dirent*)context->rsi, (size_t)context->rdx);
        break;
    case SYS_dup2: sys_dup2(context, (int)context->rdi, (int)context->rsi); break;
    case SYS_setuid: sys_setuid(context, (int)context->rdi, (int)context->rsi); break;
    case SYS_setgid: sys_setgid(context, (int)context->rdi, (int)context->rsi); break;
    default: context->rax = -ENOSYS; break;
    }
    VMM::exit_syscall_context();
}
